
module alu(
  input [15:0] a, b,
  input [7:0] nextpc,
  input [7:0] imm8,
  output reg [15:0] d,
  input [4:0] opcode
);

`include "opcodes.txt"

wire [15:0] immse = imm8[7]? {8'hFF,imm8}:{8'h00,imm8};
wire [15:0] imm = {8'h00,imm8};
//wire [15:0] sum1 = a +  immse;
wire [15:0] seim = imm8[4]? {11'h7FF, imm8[4:0]} : {11'h000, imm8[4:0]};
wire [15:0] sum = a + seim;
wire [1:0] opx = imm8[1:0];

reg [15:0] alu1, alu2;
always
case (opx)
0: alu1 <= a + b;
1: alu1 <= a - b;
default: alu1 <= 16'h0;
endcase

always
case (opx)
0: alu2 = a & b;
1: alu2 = a | b;
2: alu2 = a ^ b;
3: alu2 = ~(a|b);
endcase

always
case (opcode)
LWI:
	d <= imm;
SWI:
	d <= imm;
LI:
	d <= immse;
LUI:
	d <= {imm8, a[7:0]};
LW:
	d <= sum;
SW:	
	d <= sum;
ADDI:
	d <= sum;
ALU1:
	d <= alu1;
ALU2:
	d <= alu2;
BNE:
	d <= {8'h00, nextpc+seim[7:0]};
JALR:
	d <= {8'h00, nextpc};
default:
	d <= 16'h0000;
endcase

endmodule
